module.exports = function(grunt) {
  grunt.initConfig({
    less: {
      options: {
        compress: true
      },
      dist: {
        files: {
          'css/axiom.min.css': 'src/less/axiom.less'
        }
      }
    },
    typescript: {
      options: {
        target: 'ES5',
        sourceMap: true,
        comments: false,
        noImplicitAny: true
      },
      dist: {
        files: {
          'js/axiom.js': 'src/ts/**/*.ts'
        }
      }
    },
    uglify: {
      options: {
        compress: true,
        enclose: {}
      },
      dist: {
        files: {
          'js/axiom.min.js': 'js/axiom.js'
        }
      }
    },
    watch: {
      options: {
        atBegin: true,
        debounceDelay: 0,
        interrupt: true
      },
      css: {
        files: 'src/less/*.less',
        tasks: ['less']
      },
      js: {
        files: 'src/ts/**/*.ts',
        tasks: ['typescript']
      }
    },
    processhtml: {
      dist: {
        files: {
          'dist/index.html': 'index.html'
        }
      }
    },
    compress: {
      options: {
        archive: 'dist/static.zip'
      },
      dist: {
        files: [{
          './': ['js/axiom.min.js', 'css/**', 'templates/**', 'lib/**', '!lib/**/*.d.ts']
        }, {
          expand: true,
          cwd: 'dist/',
          src: 'index.html',
          dest: './'
        }]
      }
    }
  });

  grunt.registerTask('delete-processed',
      'Deletes the files that were created by the processhtml task.', function() {
        grunt.file.delete('dist/index.html');
      });

  grunt.loadNpmTasks('grunt-contrib-less');
  grunt.loadNpmTasks('grunt-typescript');
  grunt.loadNpmTasks('grunt-contrib-uglify');
  grunt.loadNpmTasks('grunt-contrib-watch');
  grunt.loadNpmTasks('grunt-processhtml');
  grunt.loadNpmTasks('grunt-contrib-compress');

  grunt.registerTask('default', ['less', 'typescript', 'uglify', 'processhtml', 'compress', 'delete-processed']);
};