interface AuthService {
  token: string;
  expires: Date;
  isAuthenticated(): boolean;
  authPrompt(): ng.IPromise<string>;
  authenticate(username: string, password: string): ng.IPromise<string>;
  deauthenticate(): ng.IHttpPromise<void>;
}

class AuthServiceImpl implements AuthService {

  token: string;
  expires: Date;

  private authDeferred: ng.IDeferred<string>;

  static $inject = ['$http', '$q', 'storage', 'toast'];
  constructor(
      private $http: ng.IHttpService,
      private $q: ng.IQService,
      private storage: StorageService,
      private toast: ToastService) {
    var authToken: {token: string; expires: string} = storage.get('authToken');
    if (authToken) {
      this.token = authToken.token;
      this.expires = new Date(authToken.expires);
    }
  }

  isAuthenticated() {
    return this.token != null;
  }

  authPrompt(): ng.IPromise<string> {
    if (this.authDeferred == null) {
      this.authDeferred = this.$q.defer();
      
      var $modal = $('#modal-login');
      $modal.modal('show').one('shown.bs.modal', () => {
        $('#login-username').focus();
      }).one('hide.bs.modal', () => {
        this.authDeferred = null;
      });
      
      this.authDeferred.promise.then(() => {
        $modal.modal('hide');
      });
    }
    return this.authDeferred.promise;
  }

  authenticate(username: string, password: string): ng.IPromise<string> {
    var $submit = $('#modal-login').find('[type=submit]');
    $submit.prop('disabled', true);

    this.$http.post('$axiom/auth', {
      uid: username,
      password: password
    }).success((token: {token: string; expires: string}) => {
      this.token = token.token;
      this.expires = new Date(token.expires);
      this.storage.set('authToken', token);
      this.authDeferred.resolve(this.token);
    }).error(() => {
      this.toast.error('login.errors.Failed');
    }).finally(() => {
      $submit.prop('disabled', false);
    });

    return this.authDeferred.promise;
  }

  deauthenticate(): ng.IHttpPromise<void> {
    return this.$http.delete('$axiom/auth').success(() => {
      this.storage.remove('authToken');
    });
  }
}