/// <reference path="../../lib/angularjs/angular.d.ts"/>
/// <reference path="../../lib/angularjs/angular-resource.d.ts"/>
/// <reference path="../../lib/angular-ui/angular-ui-router.d.ts"/>
/// <reference path="../../lib/angular-ui-bootstrap/angular-ui-bootstrap.d.ts"/>
/// <reference path="../../lib/bootstrap/bootstrap.d.ts"/>
/// <reference path="../../lib/lodash/lodash.d.ts"/>

angular.module('axiom', ['ui.router', 'ngResource', 'ui.bootstrap'], [
  '$provide',
  '$compileProvider',
  '$httpProvider',
  '$controllerProvider',
  '$filterProvider',
  '$urlRouterProvider',
  '$stateProvider', (
      $provide: ng.auto.IProvideService,
      $compileProvider: ng.ICompileProvider,
      $httpProvider: ng.IHttpProvider,
      $controllerProvider: ng.IControllerProvider,
      $filterProvider: ng.IFilterProvider,
      $urlRouterProvider: ng.ui.IUrlRouterProvider,
      $stateProvider: ng.ui.IStateProvider) => {

  $provide.service('currentUser', CurrentUserService);
  $provide.service('auth', AuthServiceImpl);
  $provide.service('storage', LocalStorageService);
  $provide.service('toast', ToastService);

  $provide.factory('Taskset', TasksetResourceFactory);
  $provide.factory('User', UserResourceFactory);
  $provide.factory('Solution', SolutionResourceFactory);

  $provide.provider('translation', TranslationProvider);
  
  $compileProvider.directive('tr', TranslationDirective);
  $compileProvider.directive('axMarkdown', MarkdownDirective);
  $compileProvider.directive('axDerivation', DerivationDirective);
  $compileProvider.directive('axInferenceRules', InferenceRulesDirective);
  $compileProvider.directive('axRule', InferenceRuleDirective);
  $compileProvider.directive('sequentInput', SequentInputDirective);
  $compileProvider.directive('axFormulaChooser', FormulaChooserDirective);
    
  $compileProvider.aHrefSanitizationWhitelist(/^(https?|blob):/);

  $controllerProvider.register('HudCtrl', HudCtrl);
  $controllerProvider.register('LoginCtrl', LoginCtrl);
  $controllerProvider.register('ChangePassCtrl', ChangePassCtrl);
  $controllerProvider.register('TasksetsCtrl', TasksetsCtrl);
  $controllerProvider.register('TasksetCtrl', TasksetCtrl);
  $controllerProvider.register('TasksetViewCtrl', TasksetViewCtrl);
  $controllerProvider.register('TasksetEditCtrl', TasksetEditCtrl);
  $controllerProvider.register('UsersCtrl', UsersCtrl);
  $controllerProvider.register('SolutionCtrl', SolutionCtrl);
  
  $filterProvider.register('users', UsersFilter);
  $filterProvider.register('tr', TranslationFilter);

  $httpProvider.interceptors.push(['$injector', '$q', ($injector: ng.auto.IInjectorService, $q: ng.IQService) => {
    return {
      request: (config: ng.IRequestConfig) => {
        if (config.url.indexOf('$axiom/') === 0) {
          var auth: AuthService = $injector.get('auth');
          
          config.url = '/' + config.url.substr(1);
          if (auth.isAuthenticated())
            config.params = $.extend({}, config.params, {token: auth.token});
        }
        return config;
      },
      responseError: (rejection: ng.IHttpPromiseCallbackArg<any>) => {
        if (rejection.status == 401) {
          var auth: AuthService = $injector.get('auth');
          return auth.authPrompt().then((token) => {
            var $http: ng.IHttpService = $injector.get('$http');
            rejection.config.params = $.extend({}, rejection.config.params, {token: token});
            return $http(rejection.config);
          });
        }
        return $q.reject(rejection);
      }
    };
  }]);
  
  $urlRouterProvider.otherwise('/tasksets');
  
  $stateProvider
      .state('tasksets', {
        abstract: true,
        url: '/tasksets',
        template: '<ui-view/>'
      })
        .state('tasksets.list', {
          url: '',
          controller: 'TasksetsCtrl',
          controllerAs: 'ctrl',
          templateUrl: 'templates/tasksets/index.html'
        })
        .state('tasksets.new', {
          url: '/new',
          resolve: {taskset: (): TasksetResource => null},
          controller: 'TasksetEditCtrl',
          controllerAs: 'ctrl',
          templateUrl: 'templates/tasksets/edit.html'
        })
      .state('taskset', {
        abstract: true,
        url: '/tasksets/:tasksetId',
        resolve: {
          taskset: ['$q', '$stateParams', 'translation', 'Taskset',
            ($q: ng.IQService, $stateParams: ng.ui.IStateParamsService, translation: TranslationService, Taskset: TasksetResourceClass) => {
              var def = $q.defer<TasksetResource>();
              var tasksetId = $stateParams['tasksetId'];
              var errorCallback = (result: ng.IHttpPromiseCallbackArg<TasksetResource>) => {
                if (result.status == 403) {
                  var pw = prompt(translation.get('tasksets.EnterPassword'), '');
                  if (pw == null) {
                    def.reject(result);
                    return;
                  }
                  Taskset.get({id: tasksetId, password: pw}).$promise.then(r => def.resolve(r), errorCallback);
                }
                else
                  def.reject(result);
              };
              Taskset.get({id: tasksetId}).$promise.then(r => def.resolve(r), errorCallback);
              return def.promise;
            }]
        },
        controller: 'TasksetCtrl',
        controllerAs: 'ctrl',
        template: '<ui-view/>'
      })
        .state('taskset.view', {
          url: '',
          controller: 'TasksetViewCtrl',
          controllerAs: 'ctrl',
          templateUrl: 'templates/tasksets/view.html'
        })
        .state('taskset.edit', {
          url: '/edit',
          controller: 'TasksetEditCtrl',
          controllerAs: 'ctrl',
          templateUrl: 'templates/tasksets/edit.html'
        })
        .state('taskset.solve', {
          url: '/solve/:taskId',
          resolve: {
            solution: ['$stateParams', 'Solution', ($stateParams: ng.ui.IStateParamsService, Solution: SolutionResourceClass) => {
              var taskId = $stateParams['taskId'];
              return Solution.get({taskId: taskId}).$promise;
            }]
          },
          controller: 'SolutionCtrl',
          controllerAs: 'ctrl',
          templateUrl: 'templates/solution/index.html'
        })
        .state('taskset.solutions', {
          url: '/solutions',
          resolve: {
            users: ['User', (User: UserResourceClass) => {
              return User.query().$promise;
            }]
          },
          controller: ['taskset', 'users', function(taskset: TasksetResource, users: UserResource[]) {
            this.taskset = taskset;
            this.users = users;
            this.groups = _.uniq(users.map(u => u.group)).filter(g => !!g).sort();
          }],
          controllerAs: 'ctrl',
          templateUrl: 'templates/solutions/index.html'
        })
          .state('taskset.solutions.view', {
            url: '/:userId',
            resolve: {
              tasks: ['$stateParams', 'Taskset', 'taskset',
                ($stateParams: ng.ui.IStateParamsService, Taskset: TasksetResourceClass, taskset: TasksetResource) =>
                  Taskset.get({id: taskset.id, userId: $stateParams['userId']}).$promise.then(ts => ts.tasks)
              ]
            },
            controller: ['tasks', function(tasks: Task[]) {
              this.tasks = tasks;
            }],
            controllerAs: 'ctrl',
            templateUrl: 'templates/solutions/view.html'
          })
            .state('taskset.solutions.view.derivation', {
              url: '/:taskId',
              resolve: {
                solution: ['$stateParams', 'Solution',
                  ($stateParams: ng.ui.IStateParamsService, Solution: SolutionResourceClass) =>
                    Solution.get({taskId: $stateParams['taskId'], userId: $stateParams['userId']}).$promise
                ]
              },
              controller: ['solution', 'tasks', function(solution: SolutionResource, tasks: Task[]) {
                this.solution = solution;
                this.getLemmaLabel = (lemmaTaskId: number) => _.findIndex(tasks, {id: lemmaTaskId}) + 1;
              }],
              controllerAs: 'ctrl',
              templateUrl: 'templates/solutions/derivation.html'
            })
      .state('users', {
        url: '/users',
        resolve: {
          users: ['User', (User: UserResourceClass) => User.query().$promise]
        },
        controller: 'UsersCtrl',
        controllerAs: 'ctrl',
        templateUrl: 'templates/users/index.html'
      });
  
}]).config(['translationProvider', (translationProvider: TranslationProvider) => {
  translationProvider.lang = 'et';
}]);