class HudCtrl {

  static $inject = ['currentUser', 'translation', 'auth', '$window'];
  constructor(
      private currentUser: CurrentUserService,
      private translation: TranslationService,
      private auth: AuthService,
      private $window: ng.IWindowService) {}

  get isUserPresent(): boolean {
    return this.currentUser.isPresent();
  }

  get userName(): string {
    return this.currentUser.name;
  }
  
  get langs() {
    return this.translation.langs;
  }

  get lang(): string {
    return this.translation.lang;
  }

  set lang(lang: string) {
    this.translation.lang = lang;
  }
  
  get canManageUsers(): boolean {
    return this.currentUser.isInRole(UserRole.ASSISTANT);
  }

  logout() {
    this.auth.deauthenticate().success(() => {
      this.$window.location.reload();
    });
  }
}

class LoginCtrl {
  login = {
    username: '', password: ''
  };

  static $inject = ['translation', 'auth'];
  constructor(private translation: TranslationService, private auth: AuthService) {}
  
  get langs() {
    return this.translation.langs;
  }

  get selectedLang(): string {
    return this.translation.lang;
  }

  set selectedLang(lang: string) {
    this.translation.lang = lang;
  }

  submit() {
    this.auth.authenticate(this.login.username, this.login.password);
  }
}

class ChangePassCtrl {
  changepass: {password1: string; password2: string} = {
    password1: '', password2: ''
  };

  static $inject = ['$window', 'toast', 'currentUser'];
  constructor(private $window: ng.IWindowService, private toast: ToastService,
              private currentUser: CurrentUserService) {}
  
  submit() {
    if (!this.changepass.password1 && !this.changepass.password2)
      this.toast.error('changepass.errors.EmptyPassword');
    else if (this.changepass.password1 != this.changepass.password2)
      this.toast.error('changepass.errors.PasswordsDoNotMatch');
    else
      this.currentUser.changePassword(this.changepass.password1).then(() => {
        $('#modal-changepass').modal('hide');
        this.$window.location.reload();
      });
  }
}