function SequentInputDirective(): ng.IDirective {
  return {
    restrict: 'C',
    require: '?ngModel',
    link: (scope: ng.IScope, element: JQuery, attrs: ng.IAttributes, ngModel: ng.INgModelController) => {
      var quantifiers = attrs['axSiQuantifiers'] == 'true';
      var turnstile = attrs['axSiTurnstile'] == 'true';
      var symbols = '¬&∨→~' + (quantifiers ? '∀∃' : '') + (turnstile ? '⊢' : '');
      
      function insertSymbol(symbol: string) {
        var el = <HTMLInputElement>element.get(0);
        var val: string = element.val();
        var selStart = el.selectionStart;
        element.val(val.substr(0, selStart) + symbol + val.substr(el.selectionEnd));
        el.selectionStart = el.selectionEnd = selStart + 1;

        if (ngModel)
          ngModel.$setViewValue(element.val());
      }
      
      element.on('keydown', (ev: JQueryKeyEventObject) => {
        if (ev.keyCode >= 49 && ev.keyCode < 49 + symbols.length && !ev.shiftKey) {
          insertSymbol(symbols[ev.keyCode - 49]);
          ev.preventDefault();
        }
      });
      
      var clickingButton = false;

      var buttons = $('<div class="sequent-input-buttons btn-group btn-group-xs"></div>');
      symbols.split('').forEach(s => {
        $('<button tabindex="-1" type="button" class="btn btn-default sequent"></button>')
            .text(s)
            .mousedown(() => { clickingButton = true; })
            .click(() => { insertSymbol(s); })
            .appendTo(buttons);
      });
      buttons.insertAfter(element);
      
      element.on('focus', () => {
        buttons.fadeIn('fast');
      });
      
      element.on('blur', () => {
        if (!clickingButton)
          buttons.fadeOut('fast');
        clickingButton = false;
      });
    }
  };
}

interface FormulaChooserScope extends ng.IScope {
  formulas: string[];
  chosen: number;
  choose(index: number): void;
}

function FormulaChooserDirective(): ng.IDirective {
  return {
    restrict: 'E',
    scope: {
      formulas: '=',
      chosen: '='
    },
    template: '<div ng-repeat="formula in formulas"><span class="formula" ng-click="choose($index)" ng-class="{active: $index == chosen}">{{formula}}</span></div>',
    link: (scope: FormulaChooserScope, element: JQuery, attrs: ng.IAttributes) => {
      scope.choose = index => {
        scope.chosen = index;
      };
    }
  };
}