interface ToastMessageAdder {
  (message: string, args?: any[]): ToastMessageAdder;
}

class ToastService {
  private toastsEl: JQuery;
  
  static $inject = ['$timeout', 'translation'];
  constructor(private $timeout: ng.ITimeoutService, private translation: TranslationService) {
    this.toastsEl = $('#toasts');
  }

  error(message: string, args?: any[]): ToastMessageAdder {
    return this.message('error', message, args);
  }
  
  info(message: string, args?: any[]) {
    return this.message('info', message, args);
  }
  
  private message(type: string, message: string, args?: any[]): ToastMessageAdder {
    this.clear();
    var toast = this.addToast(type);
    var adder = (message: string, args?: any[]) => {
      var parts = message.split('$');
      message = parts.shift();
      toast.append($('<span/>').text(this.translation.get(message, parts.concat(args))));
      return adder;
    };
    return adder(message, args);
  }
  
  clear() {
    this.toastsEl.empty();
  }
  
  private addToast(type: string): JQuery {
    var toast = $('<div/>').addClass(type).hide();
    this.toastsEl.append(toast);
    toast.fadeIn('fast');
    this.$timeout(() => {
      toast.fadeOut('fast', () => { toast.remove(); });
    }, 6000);
    return toast;
  }
}