class TranslationProvider {
  lang: string;

  $get = ['$injector', ($injector: ng.auto.IInjectorService) => {
    var translation: TranslationService = $injector.instantiate(TranslationService);
    translation.lang = this.lang;
    return translation;
  }];
}

class TranslationService {
  public static EVT_LANG_CHANGE = 'axTranslationLangChange';
  
  private _lang: string;
  
  public langs: {[lang: string]: string} = {};
  
  private messages: {[key: string]: string} = {};
  
  static $inject = ['$http', '$rootScope'];
  constructor(private $http: ng.IHttpService, private $rootScope: ng.IRootScopeService) {}
  
  get lang(): string {
    return this._lang;
  }
  
  set lang(lang: string) {
    this.fetch(lang).then(() => {
      this.$rootScope.$broadcast(TranslationService.EVT_LANG_CHANGE);
    });
  }

  get(key: string, args?: string[]): string {
    if (!this._lang)
      return '';
    if (key in this.messages)
      return this.format(this.messages[key], args);
    return key;
  }
  
  private format(s: string, args: string[]): string {
    return s.replace(/{(\d+)}/g, (m: string, d: number) => d in args ? args[d] : m);
  }
  
  private fetch(lang: string) {
    return this.$http.get('$axiom/translations', {
      params: {
        lang: lang
      }
    }).success((data: {lang: string; langName: string; langs: {[key: string]: string}; messages: {[key: string]: string}}) => {
      this._lang = data.lang;
      angular.copy(data.langs, this.langs);
      angular.copy(data.messages, this.messages);
    });
  }
  
}

TranslationDirective.$inject = ['translation'];
function TranslationDirective(translation: TranslationService): ng.IDirective {
  return {
    restrict: 'A',
    link: (scope: ng.IScope, element: JQuery, attrs: ng.IAttributes) => {
      var setter = element.text.bind(element);
      if (element.is('input'))
        setter = element.attr.bind(element, 'placeholder');
      else if (!element.is(':empty'))
        setter = element.attr.bind(element, 'title');
      
      var update = () => {
        var args = attrs['tr'].split('$');
        var key = args.shift();
        var text = translation.get(key, args);
        setter(text);
      };
      scope.$on(TranslationService.EVT_LANG_CHANGE, update);
      attrs.$observe('tr', update);
    }
  };
}

TranslationFilter.$inject = ['translation'];
function TranslationFilter(translation: TranslationService) {
  return (key: string) => translation.get(key);
}