package ee.ut.axiom;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.concurrent.ExecutionException;

import org.glassfish.grizzly.GrizzlyFuture;
import org.glassfish.grizzly.http.server.HttpServer;
import org.glassfish.grizzly.http.server.NetworkListener;
import org.glassfish.grizzly.http.server.StaticHttpHandler;
import org.glassfish.grizzly.websockets.WebSocketAddOn;
import org.glassfish.grizzly.websockets.WebSocketEngine;
import org.glassfish.jersey.grizzly2.httpserver.GrizzlyHttpServerFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Main {
  private static final String DEFAULT_HOST = "127.0.0.1";
  private static final int DEFAULT_PORT = 8000;
  private static final String BACKEND_PATH = "/axiom";

  private static final Logger log = LoggerFactory.getLogger(Main.class);
  
  public static HttpServer createServer() throws IOException, URISyntaxException {
    Path homePath = detectHome();
    if (homePath == null) {
      log.error("Please specify the ee.ut.axiom.home system property.");
      return null;
    }

    Path staticContentPath = homePath.resolve(System.getProperty("ee.ut.axiom.staticDirName", "static"));
    if (!staticContentPath.toFile().isDirectory()) {
      log.error("The static content directory was not found in the home directory.");
      return null;
    }

    AxiomApplication axiomApp = new AxiomApplication(homePath);

    String host = System.getProperty("ee.ut.axiom.host", DEFAULT_HOST);
    int port = Integer.getInteger("ee.ut.axiom.port", DEFAULT_PORT);
    URI uri = new URI("http", null, host, port, BACKEND_PATH, null, null);

    HttpServer server = GrizzlyHttpServerFactory.createHttpServer(uri, axiomApp, false);

    server.getServerConfiguration().addHttpHandler(new StaticHttpHandler(staticContentPath.toString()), "/");

    NetworkListener firstListener = server.getListeners().iterator().next();
    firstListener.getFileCache().setEnabled(false);
    firstListener.registerAddOn(new WebSocketAddOn());
    
    WebSocketEngine.getEngine().register("", BACKEND_PATH + "/solve/*", new SolvingWSApplication(axiomApp));

    server.start();
    return server;
  }

  private static Path detectHome() throws IOException, URISyntaxException {
    String givenHome = System.getProperty("ee.ut.axiom.home");
    if (givenHome != null)
      return Paths.get(givenHome).toRealPath();

    Path codeSourcePath = Paths.get(Main.class.getProtectionDomain().getCodeSource().getLocation().toURI());
    if (codeSourcePath.toFile().isFile())
      return codeSourcePath.getParent();
    
    return null;
  }

  public static void main(String[] args) throws IOException, URISyntaxException {
    HttpServer server = createServer();
    
    if (server == null)
      System.exit(1);
    
    Runtime.getRuntime().addShutdownHook(new Thread(() -> {
      GrizzlyFuture<HttpServer> future = server.shutdown();
      try {
        future.get();
        System.out.println("Server shutdown complete.");
      }
      catch (InterruptedException | ExecutionException e) {
        e.printStackTrace();
      }
    }));

    try {
      Thread.sleep(Long.MAX_VALUE);
    }
    catch (InterruptedException ignored) {}
  }
}

