package ee.ut.axiom;

import java.util.HashMap;
import java.util.Map;

import org.glassfish.grizzly.http.HttpRequestPacket;
import org.glassfish.grizzly.websockets.DataFrame;
import org.glassfish.grizzly.websockets.ProtocolHandler;
import org.glassfish.grizzly.websockets.WebSocket;
import org.glassfish.grizzly.websockets.WebSocketApplication;
import org.glassfish.grizzly.websockets.WebSocketListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.google.gson.Gson;

import ee.ut.axiom.sequentCalculus.ParseException;
import ee.ut.axiom.sequentCalculus.RuleApplicationConstraintException;
import ee.ut.axiom.sequentCalculus.RuleNotApplicableException;
import ee.ut.axiom.sequentCalculus.VariableRenameClashException;
import ee.ut.axiom.util.ParseExceptionMessage;

/**
 * @author Dmitri Gabbasov
 */
public class SolvingWSApplication extends WebSocketApplication {
  Logger log = LoggerFactory.getLogger(SolvingWSApplication.class);
  
  private AxiomApplication axiomApp;

  public SolvingWSApplication(AxiomApplication axiomApp) {
    this.axiomApp = axiomApp;
  }

  @Override
  public WebSocket createSocket(ProtocolHandler handler, HttpRequestPacket requestPacket, WebSocketListener... listeners) {
    return new SolvingWebSocket(axiomApp.getEntityManagerFactory(), handler, requestPacket, listeners);
  }

  public void onConnect(WebSocket socket) {
    super.onConnect(socket);
    log.info("WebSocket {} connected", socket);
  }

  @Override
  public void onClose(WebSocket socket, DataFrame frame) {
    super.onClose(socket, frame);
    SolvingWebSocket solvingWebSocket = (SolvingWebSocket) socket;
    
    log.info("WebSocket {} disconnected", socket);
  }

  @Override
  protected boolean onError(WebSocket webSocket, Throwable t) {
    log.error("Unexpected exception", t);
    
    Map<String, String> error = new HashMap<>();
    error.put("error", t.getClass().getSimpleName());
    
    if (t instanceof ParseException)
      error.put("message", ParseExceptionMessage.of((ParseException) t));
    else if (t instanceof RuleNotApplicableException)
      error.put("message", "solution.errors.RuleNotApplicable");
    else if (t instanceof VariableRenameClashException)
      error.put("message", "solution.errors.ChosenVarMustBeFree");
    else if (t instanceof RuleApplicationConstraintException)
      error.put("message", "solution.errors.VarMustBeFresh");
    else
      error.put("message", t.getMessage());
    
    webSocket.send(new Gson().toJson(error));
    return false;
  }
}
