package ee.ut.axiom;

import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.ws.rs.ForbiddenException;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.core.UriInfo;
import javax.ws.rs.ext.ExceptionMapper;
import javax.ws.rs.ext.Provider;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Dmitri Gabbasov
 */
@Provider
public class UnexpectedExceptionMapper implements ExceptionMapper<Exception> {
  private static Logger log = LoggerFactory.getLogger(UnexpectedExceptionMapper.class);
  
  @Inject
  UriInfo uriInfo;
  
  @Inject
  javax.inject.Provider<EntityManager> em;
  
  @Inject
  javax.inject.Provider<SecurityContext> secCon;

  @Override
  public Response toResponse(Exception exception) {
    if (em.get().getTransaction().isActive()) {
      log.debug("Marking current transaction for rollback");
      em.get().getTransaction().rollback();
    }
    
    if (exception instanceof WebApplicationException) {
      if (((WebApplicationException) exception).getResponse().getStatus() / 100 == 4)
        log.warn("WebApplicationException: {} ({})", exception.getMessage(), uriInfo.getPath());
      else
        log.error("Unexpected exception", exception);
      
      if (exception instanceof ForbiddenException && secCon.get().getUserPrincipal() == null) {
        return Response.status(Response.Status.UNAUTHORIZED).build();
      }
      
      return ((WebApplicationException) exception).getResponse();
    }
    
    if (exception instanceof NoResultException) {
      return Response.status(Response.Status.NOT_FOUND).build();
    }

    log.error("Unexpected exception", exception);
    
    return Response.serverError().build();
  }
}
