package ee.ut.axiom.models;

import java.security.Principal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.xml.bind.annotation.XmlTransient;

import org.apache.commons.codec.digest.DigestUtils;
import org.hibernate.validator.constraints.NotBlank;

/**
 * @author Dmitri Gabbasov
 */
@Entity
public class User implements Principal {
  public static final String STUDENT = "STUDENT";
  public static final String ASSISTANT = "ASSISTANT";
  public static final String INSTRUCTOR = "INSTRUCTOR";
  public static final String ADMIN = "ADMIN";
  
  private static final List<String> rolesByPriority = Arrays.asList(STUDENT, ASSISTANT, INSTRUCTOR, ADMIN);
  
  public static final String ANY = STUDENT;
  
  @Id
  @Column(length = 30)
  public String uid;
  
  @XmlTransient
  @Column(nullable = false, length = 64)
  public String passwordHash;
  
  @NotBlank(message = "users.errors.EmptyName")
  @Column(nullable = false, length = 100)
  public String name;
  
  @Column(length = 10)
  public String sbn;
  
  @Column(name = "groupName", length = 50)
  public String group;
  
  @NotBlank
  @Column(length = 15)
  public String role;
  
  @XmlTransient
  @OneToMany(cascade = CascadeType.ALL, mappedBy = "user", orphanRemoval = true)
  public List<AuthToken> activeTokens = new ArrayList<>();
  
  @XmlTransient
  @OneToMany(cascade = CascadeType.ALL, mappedBy = "user", orphanRemoval = true)
  public List<Solution> solutions = new ArrayList<>();

  public User() { }

  @Override
  public String getName() {
    return uid;
  }
  
  public boolean isInRole(String role) {
    return rolesByPriority.indexOf(role) <= rolesByPriority.indexOf(this.role);
  }
  
  public static String hashPassword(String password) {
    return DigestUtils.sha256Hex(password);
  }

  @Override
  public String toString() {
    return "User{" +
        "uid='" + uid + '\'' +
        ", passwordHash='" + passwordHash + '\'' +
        ", name='" + name + '\'' +
        ", sbn='" + sbn + '\'' +
        ", group='" + group + '\'' +
        ", role=" + role +
        '}';
  }
}
