package ee.ut.axiom.repositories;

import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;
import javax.inject.Inject;
import javax.persistence.EntityManager;

import org.jvnet.hk2.annotations.Service;

import com.google.common.base.Preconditions;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;

import ee.ut.axiom.models.AuthToken;
import ee.ut.axiom.models.User;

import static java.util.stream.Collectors.toList;

/**
 * @author Dmitri Gabbasov
 */
@Service
public class UserRepository {

  private EntityManager em;

  @Inject
  public UserRepository(EntityManager em) {
    this.em = em;
  }

  public Optional<AuthToken> findAuthToken(String token) {
    List<AuthToken> result = em.createQuery("select at from AuthToken at where at.token=:token and at.expires > CURRENT_TIMESTAMP", AuthToken.class)
        .setParameter("token", token).getResultList();
    return result.isEmpty() ? Optional.empty() : Optional.of(result.get(0));
  }
  
  public Optional<User> findByToken(String token) {
    List<User> result = em.createQuery("select at.user from AuthToken at where at.token=:token and at.expires > CURRENT_TIMESTAMP", User.class)
        .setParameter("token", token).getResultList();
    return result.isEmpty() ? Optional.empty() : Optional.of(result.get(0));
  }

  public Optional<User> findByCredentials(String uid, String password) {
    List<User> result = em.createQuery("select u from User u where u.uid=:uid and u.passwordHash=:hash", User.class)
        .setParameter("uid", uid)
        .setParameter("hash", User.hashPassword(password))
        .getResultList();
    return result.isEmpty() ? Optional.empty() : Optional.of(result.get(0));
  }
  
  public AuthToken createAuthTokenFor(User user) {
    AuthToken at = AuthToken.createRandomToken(user, Date.from(Instant.now().plus(4, ChronoUnit.HOURS)));
    user.activeTokens.add(at);
    em.persist(at);
    return at;
  }

  public List<User> getAll() {
    return em.createQuery("select u from User u", User.class).getResultList();
  }

  public User getByUid(String uid) {
    return em.createQuery("select u from User u where u.uid=:uid", User.class)
        .setParameter("uid", uid).getSingleResult();
  }
  public Optional<User> findByUid(String uid) {
    return Optional.ofNullable(em.find(User.class, uid));
  }
  
  public User save(User user) {
    em.persist(user);
    return user;
  }

  public void deleteUser(User user) {
    em.remove(user);
  }
}
