package ee.ut.axiom.resources;

import java.security.Principal;
import java.util.Iterator;
import java.util.Optional;
import javax.annotation.security.RolesAllowed;
import javax.inject.Inject;
import javax.ws.rs.DELETE;
import javax.ws.rs.ForbiddenException;
import javax.ws.rs.GET;
import javax.ws.rs.NotFoundException;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.SecurityContext;

import com.google.common.collect.Lists;

import ee.ut.axiom.models.AuthToken;
import ee.ut.axiom.models.User;
import ee.ut.axiom.repositories.UserRepository;

/**
 * @author Dmitri Gabbasov
 */
@Path("/auth")
@Produces(MediaType.APPLICATION_JSON)
public class AuthResource {
  @Inject
  UserRepository users;

  @Inject
  SecurityContext secCon;
  
  public static class AuthParams {
    public String uid;
    public String password;
  }

  @GET
  public User me() {
    Principal user = secCon.getUserPrincipal();
    if (user == null)
      throw new ForbiddenException();

    return users.getByUid(user.getName());
  }

  @POST
  public AuthToken authenticate(AuthParams params) {
    Optional<User> user = users.findByCredentials(params.uid, params.password);
    Optional<AuthToken> at = user.map(users::createAuthTokenFor);
    return at.orElseThrow(NotFoundException::new);
  }

  @DELETE
  @RolesAllowed(User.ANY)
  public void deauthenticate(@QueryParam("token") String token) {
    Iterator<AuthToken> iterator = me().activeTokens.iterator();
    while (iterator.hasNext())
      if (iterator.next().token.equals(token))
        iterator.remove();
  }
  
  @POST
  @Path("/changepw")
  @RolesAllowed(User.ANY)
  public void changePassword(@QueryParam("password") String password) {
    if (password != null && !password.isEmpty()) {
      User me = me();
      me.passwordHash = User.hashPassword(password);
      me.activeTokens.clear();
    }
  }
}
