package ee.ut.axiom.resources;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.security.RolesAllowed;
import javax.inject.Inject;
import javax.persistence.NoResultException;
import javax.ws.rs.ForbiddenException;
import javax.ws.rs.GET;
import javax.ws.rs.NotFoundException;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.SecurityContext;

import com.google.gson.Gson;

import ee.ut.axiom.models.User;
import ee.ut.axiom.repositories.SolutionRepository;
import ee.ut.axiom.repositories.TasksetRepository;
import ee.ut.axiom.sequentCalculus.InferenceRule;
import ee.ut.axiom.views.SimpleSolution;

/**
 * @author Dmitri Gabbasov
 */
@Path("solutions")
@Produces(MediaType.APPLICATION_JSON)
public class SolutionResource {
  
  @Inject
  SolutionRepository solutions;
  
  @Inject
  TasksetRepository tasksets;
  
  @Inject
  SecurityContext secCon;
  
  @GET @Path("{taskId}")
  @RolesAllowed(User.STUDENT)
  public SimpleSolution get(@PathParam("taskId") int taskId, @QueryParam("userId") String userId) {
    if (userId != null && !secCon.isUserInRole(User.ASSISTANT))
      throw new ForbiddenException();
    
    if (userId == null)
      userId = secCon.getUserPrincipal().getName();

    try {
      return new SimpleSolution(solutions.getSolution(taskId, userId));
    }
    catch (NoResultException e) {
      return SimpleSolution.empty(tasksets.findTask(taskId).orElseThrow(NotFoundException::new).sequent);
    }
  }
  
  @GET @Path("rules")
  @RolesAllowed(User.STUDENT)
  public String rules() {
    List<Map<String, String>> rules = new ArrayList<>();
    for (InferenceRule desc : InferenceRule.values()) {
      Map<String, String> descMap = new HashMap<>();
      descMap.put("top", desc.top);
      descMap.put("bottom", desc.bottom);
      descMap.put("symbol", desc.symbol);
      rules.add(descMap);
    }
    return new Gson().toJson(rules);
  }
}
