package ee.ut.axiom.resources;

import java.util.List;
import javax.annotation.security.RolesAllowed;
import javax.inject.Inject;
import javax.validation.Valid;
import javax.ws.rs.DELETE;
import javax.ws.rs.ForbiddenException;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.SecurityContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ee.ut.axiom.models.Taskset;
import ee.ut.axiom.models.User;
import ee.ut.axiom.repositories.TasksetRepository;
import ee.ut.axiom.views.SimpleTaskset;
import ee.ut.axiom.views.TasksetWithSolutionInfo;

/**
 * @author Dmitri Gabbasov
 */
@Path("/tasksets")
@Produces(MediaType.APPLICATION_JSON)
public class TasksetResource {
  private static Logger log = LoggerFactory.getLogger(TasksetResource.class);

  @Inject
  TasksetRepository tasksetRepo;

  @Inject
  SecurityContext secCon;

  @GET
  @RolesAllowed(User.STUDENT)
  public List<SimpleTaskset> list() {
    if (secCon.isUserInRole(User.INSTRUCTOR))
      return tasksetRepo.getAllTasksets();
    
    return tasksetRepo.getOpenTasksets();
  }
  
  @GET @Path("{id}")
  @RolesAllowed(User.STUDENT)
  public TasksetWithSolutionInfo getTaskset(@PathParam("id") int id, @QueryParam("userId") String userId,
                                            @QueryParam("password") String password) {
    if (userId != null && !secCon.isUserInRole(User.ASSISTANT))
      throw new ForbiddenException();
    if (userId == null)
      userId = secCon.getUserPrincipal().getName();

    Taskset taskset = tasksetRepo.getTasksetById(id);
    if (!secCon.isUserInRole(User.ASSISTANT) && taskset.password != null && !taskset.password.equals(password))
      throw new ForbiddenException();
    
    return tasksetRepo.getTasksetWithSolutions(id, userId);
  }
  
  @GET @Path("{id}/edit")
  @RolesAllowed(User.INSTRUCTOR)
  public Taskset editTaskset(@PathParam("id") int id) {
    return tasksetRepo.getTasksetById(id);
  }
  
  @POST
  @RolesAllowed(User.INSTRUCTOR)
  public void createTaskset(@Valid Taskset taskset) {
    saveTaskset(taskset);
  }
  
  @POST @Path("{id}")
  @RolesAllowed(User.INSTRUCTOR)
  public void saveTaskset(@Valid Taskset taskset) {
    tasksetRepo.save(taskset);
  }
  
  @DELETE @Path("{id}")
  @RolesAllowed(User.INSTRUCTOR)
  public void delete(@PathParam("id") int tasksetId) {
    tasksetRepo.deleteTaskset(tasksetId);
  }
}
