package ee.ut.axiom.resources;

import java.util.Map;
import javax.inject.Inject;
import javax.ws.rs.GET;
import javax.ws.rs.NotFoundException;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

import com.google.common.collect.Maps;
import com.google.gson.Gson;

import ee.ut.axiom.MessagesService;

/**
 * @author Dmitri Gabbasov
 */
@Path("/translations")
@Produces(MediaType.APPLICATION_JSON)
public class TranslationsResource {
  @Inject
  MessagesService messages;
  
  public static class Translation {
    public String lang;
    public String langName;
    public Map<String, String> langs;
    public Map<String, String> messages;
  }
  
  @GET
  public String get(@QueryParam("lang") String lang) {
    Map<String, MessagesService.Language> langs = messages.getLanguages();

    MessagesService.Language language = lang == null ? null : langs.get(lang);
    if (language == null)
      throw new NotFoundException();

    Translation trans = new Translation();
    trans.lang = language.id;
    trans.langName = language.name;
    trans.langs = Maps.asMap(langs.keySet(), key -> langs.get(key).name);
    trans.messages = language.messages;
    return new Gson().toJson(trans);
  }
}
