package ee.ut.axiom.sequentCalculus;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import com.google.common.collect.Sets;

/**
 * @author Dmitri Gabbasov
 */
public class Sequent {
  public List<Formula> antecedent = new ArrayList<>();
  public Formula succedent;
  
  public static Sequent parse(String sequent) {
    return new Parser(sequent).parseSequent();
  }

  public Formula removeLastAntecedentFormula() {
    return antecedent.isEmpty() ? null : antecedent.remove(antecedent.size() - 1);
  }
  
  public Set<Character> getVariables() {
    return antecedent.stream().map(Formula::getVariables).reduce(succedent.getVariables(), Sets::union);
  }
  
  public boolean isPropositional() {
    return antecedent.stream().allMatch(Formula::isPropositional) && succedent.isPropositional();
  }
  
  public boolean isDerivable() {
    return Formulas.isTautology(asFormula());
  }

  private Formula asFormula() {
    Optional<Formula> antecedent = this.antecedent.stream().reduce(Formulas::and);
    return antecedent.isPresent() ? Formulas.imply(antecedent.get(), succedent) : succedent;
  }

  public Sequent copy() {
    Sequent copy = new Sequent();
    copy.antecedent = antecedent.stream().map(Formula::copy).collect(Collectors.toList());
    copy.succedent = succedent.copy();
    return copy;
  }

  @Override
  public String toString() {
    return String.join(String.valueOf(Tokens.COMMA),
        antecedent.stream().map(Formula::toString).collect(Collectors.toList())) + Tokens.TURNSTILE + succedent;
  }

  public String detectAxiom() {
    if (antecedent.stream().anyMatch(succedent::equals))
      return "Ax";
    
    if (succedent instanceof Formulas.Predicate) {
      Formulas.Predicate predicate = (Formulas.Predicate) succedent;
      if (predicate.name == '=' && predicate.args[0] == predicate.args[1])
        return "Re";
    }
    
    if (antecedent.stream().allMatch(f -> f instanceof Formulas.Predicate && ((Formulas.Predicate) f).name == '=')
        && succedent instanceof Formulas.Implication) {
      Formulas.Implication impl = (Formulas.Implication) succedent;
      if (impl.left instanceof Formulas.Predicate && impl.right instanceof Formulas.Predicate) {
        Formulas.Predicate left = (Formulas.Predicate) impl.left;
        Formulas.Predicate right = (Formulas.Predicate) impl.right;
        int n = antecedent.size();
        if (right.args.length == n && left.args.length == n) {
          boolean ok = true;
          for (int i = 0; i < n && ok; ++i) {
            Formulas.Predicate eq = (Formulas.Predicate) antecedent.get(i);
            if (eq.args[0] != left.args[i] || eq.args[1] != right.args[i])
              ok = false;
          }
          if (ok)
            return "=P";
        }
      }
    }
    
    return null;
  }
  
  public boolean unify(Sequent subject) {
    int extra = subject.antecedent.size() - antecedent.size();
    if (extra < 0)
      return false;
    
    Map<Character, String> subst = new HashMap<>();
    boolean allUnify = Formulas.unify(succedent, subject.succedent, subst);
    
    for (int i = 0; i < antecedent.size() && allUnify; ++i)
      allUnify = Formulas.unify(antecedent.get(i), subject.antecedent.get(extra + i), subst);

    return allUnify;
  }

}
