package ee.ut.axiom.util;

import com.google.common.base.Joiner;
import com.google.common.collect.ObjectArrays;

import ee.ut.axiom.sequentCalculus.BoundVariableClashException;
import ee.ut.axiom.sequentCalculus.ExcessiveParenthesisException;
import ee.ut.axiom.sequentCalculus.ParseException;
import ee.ut.axiom.sequentCalculus.PredicateArityMismatchException;

/**
 * @author Dmitri Gabbasov
 */
public class ParseExceptionMessage {

  private static final Joiner JOINER = Joiner.on('$');
  
  public static String of(ParseException e) {
    if (e instanceof PredicateArityMismatchException)
      return message("PredicateArityMismatch", ((PredicateArityMismatchException) e).predicate);
    
    if (e instanceof ExcessiveParenthesisException)
      return message("ExcessiveParenthesis", e.position);
    
    if (e instanceof BoundVariableClashException) {
      BoundVariableClashException clashException = (BoundVariableClashException) e;
      return message(clashException.free ? "BoundVariableClashWithFree" : "BoundVariableClash", clashException.varName);
    }
    
    if (e.position != null && e.actual != null && e.expected != null)
      return message("ExpectingDifferentSymbol", e.position, e.actual, e.expected);
    
    if (e.position != null && e.actual != null)
      return message("UnexpectedSymbol", e.position, e.actual);
    
    return message("ParseError");
  }
  
  private static String message(String key, Object... args) {
    return JOINER.join(ObjectArrays.concat("sequentCalculus." + key, args));
  }
}
