package ee.ut.axiom.sequentCalculus;

import java.util.Arrays;

import org.junit.Test;

import static ee.ut.axiom.sequentCalculus.Sequents.asciiSequent;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.assertThat;

/**
 * @author Dmitri Gabbasov
 */
public class InferenceRulesTest {
  @Test(expected = VariableRenameClashException.class)
  public void universalQLeftTest() {
    applyRule("@x@yP(x,y)|-A", InferenceRule.UNIVERSALQ_LEFT, "y");
  }
  
  @Test
  public void universalQRightTest() {
    assertRuleResult("Q(x)|-@x@yP(x,y)", InferenceRule.UNIVERSALQ_RIGHT, null, "Q(x)|-@yP(z,y)");
  }

  @Test
  public void existentialQLeftTest() {
    assertRuleResult("#x#yP(x,y)|-Q(x)", InferenceRule.EXISTENTIALQ_LEFT, null, "#yP(z,y)|-Q(x)");
  }

  @Test(expected = VariableRenameClashException.class)
  public void existentialQRightTest() {
    applyRule("|-#x#yP(x,y)", InferenceRule.EXISTENTIALQ_RIGHT, "y");
  }

  private static String[] applyRule(String sequent, InferenceRule rule, String input) {
    return Arrays.stream(rule.apply(asciiSequent(sequent), input)).map(Sequent::toString).toArray(String[]::new);
  }

  private static void assertRuleResult(String sequent, InferenceRule rule, String input, String... result) {
    String[] resultSequents = applyRule(sequent, rule, input);
    String[] expectedSequents = Arrays.stream(result).map(Sequents::asciiSequent).map(Sequent::toString).toArray(String[]::new);
    assertThat(resultSequents, is(arrayContaining(expectedSequents)));
  }
}
