package ee.ut.axiom.sequentCalculus;

import org.junit.Test;

import static ee.ut.axiom.sequentCalculus.Sequents.asciiSequent;

/**
 * @author Dmitri Gabbasov
 */
public class ParserTest {
  @Test(expected = BoundVariableClashException.class)
  public void testBoundVariableClash() {
    asciiSequent("|-@x@xP(x)");
  }
  
  @Test(expected = BoundVariableClashException.class)
  public void testBoundVariableClashWithFree() {
    asciiSequent("|-P(x)&@xP(x)");
  }
  
  @Test(expected = PredicateArityMismatchException.class)
  public void testPredicateArityMismatch() {
    asciiSequent("P(x)|-P(x,y)");
  }
  
  @Test(expected = ExcessiveParenthesisException.class)
  public void testExcessiveOuterParenthesis() {
    asciiSequent("|-(A&B|C)");
  }
  
  @Test(expected = ExcessiveParenthesisException.class)
  public void testExcessiveDoubleParenthesis() {
    asciiSequent("|-A&((B|C))");
  }
  
  @Test(expected = ParseException.class)
  public void testNegationWithInfixPredicate() {
    asciiSequent("|-!x=x");
  }
  
  @Test
  public void testNegationWithGroupedInfixPredicate() {
    asciiSequent("|-!(x=x)");
  }
  
  @Test(expected = ParseException.class)
  public void testUniversalQWithInfixPredicate() {
    asciiSequent("|-@xx=x");
  }
  
  @Test
  public void testUniversalQWithGroupedInfixPredicate() {
    asciiSequent("|-@x(x=x)");
  }
  
  @Test(expected = ParseException.class)
  public void testExistentialQWithInfixPredicate() {
    asciiSequent("|-#xx=x");
  }
  
  @Test
  public void testExistentialQWithGroupedInfixPredicate() {
    asciiSequent("|-#x(x=x)");
  }
}
