package ee.ut.axiom.sequentCalculus;

import org.junit.Test;

import static ee.ut.axiom.sequentCalculus.Sequents.*;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

/**
 * @author Dmitri Gabbasov
 */
public class UnificationTest {
  @Test
  public void unifyTest() {
    Sequent contradiction = asciiSequent("A,!A|-B");
    assertUnifies(contradiction, asciiSequent("A,!A|-B"));
    assertUnifies(contradiction, asciiSequent("X,!X|-Y"));
    assertUnifies(contradiction, asciiSequent("!X,!!X|-Y"));
    assertUnifies(contradiction, asciiSequent("X&Y,!(X&Y)|-Z"));
    assertUnifies(contradiction, asciiSequent("P(x),!P(x)|-Q(x)"));
    assertUnifies(contradiction, asciiSequent("@x(P(x)|Q(x)),!@x(P(x)|Q(x))|-R(x)"));
    assertUnifies(contradiction, asciiSequent("x=y,!(x=y)|-A"));
    assertDoesntUnify(contradiction, asciiSequent("!A,A|-B"));
    assertDoesntUnify(contradiction, asciiSequent("!X,X|-Y"));
    assertDoesntUnify(contradiction, asciiSequent("X|-X"));
    assertDoesntUnify(contradiction, asciiSequent("|-X"));
    assertDoesntUnify(contradiction, asciiSequent("|-X|!X"));

    Sequent symmetry = asciiSequent("x=y|-y=x");
    assertUnifies(symmetry, asciiSequent("x=y|-y=x"));
    assertUnifies(symmetry, asciiSequent("a=b|-b=a"));
    assertUnifies(symmetry, asciiSequent("a=a|-a=a"));
    assertDoesntUnify(symmetry, asciiSequent("A|-B"));
    assertDoesntUnify(symmetry, asciiSequent("x=y|-x=x"));
    
    assertUnifies(asciiSequent("|-A|@xP(x)"), asciiSequent("|-#xP(x)|@xP(x)"));
    assertUnifies(asciiSequent("|-@xP(x)|#x!P(x)"), asciiSequent("X,Y|-@xP(x)|#y!P(y)"));
    assertUnifies(asciiSequent("|-@x@y@z(x=y&y=z>x=z)"), asciiSequent("|-@z@a@b(z=a&a=b>z=b)"));
    
    assertDoesntUnify(asciiSequent("@xQ(x),P(x)|-R(x)"), asciiSequent("@xQ(x),P(x)|-R(y)"));
  }

  private void assertUnifies(Sequent pattern, Sequent subject) {
    assertTrue(pattern + " failed to unify with " + subject, pattern.unify(subject));
  }

  private void assertDoesntUnify(Sequent pattern, Sequent subject) {
    assertFalse(pattern + " wrongly unified with " + subject, pattern.unify(subject));
  }

}
